@@ -54,7 +54,7 @@ prepare_results.pca <- function(obj) {
   vars <- vars %>% left_join(tmp, by = c("varname", "Type", "Class", "Axis"))
   
   ## Cos2
-  tmp <- inertia$col.rel / 100
+  tmp <- abs(inertia$col.rel) / 100
   tmp <- tmp %>% mutate(varname = rownames(tmp),
                         Type = "Active", Class = "Quantitative")
   tmp <- tmp %>% gather(Axis, Cos2, starts_with("Axis")) %>%
@@ -92,7 +92,7 @@ prepare_results.pca <- function(obj) {
   ind <- ind %>% left_join(tmp, by = c("Name", "Type", "Axis"))
   
   ## Individuals Cos2
-  tmp <- inertia$row.rel / 100
+  tmp <- abs(inertia$row.rel) / 100
   tmp$Name <- rownames(tmp)
   tmp$Type <- "Active"
   tmp <- tmp %>%
